
###############################################################################
# EXAMPLE VARNISH CONFIGURATION
#
# sections you should customize to match your configuration
# are marked with the word "CUSTOMIZE"
# define the available back-ends.
# CUSTOMIZE this section to point to your packagers.
#
# in this example, we have 2 packagers hosted at packager1.example.com
# and packager2.example.com
# 
 
backend b1 {
    .host = packager1.example.com;
    .probe = {
        .url = "/";
        .timeout = 150 ms;
        .interval = 10s;
        .window = 6;
        .threshold = 5;
    }
}
 
backend b2 {
    .host = packager2.example.com;
    .probe = {
        .url = "/";
        .timeout = 150 ms;
        .interval = 10s;
        .window = 6;
        .threshold = 5;
    }
}

# define the hash director that we is used for m3u8 (HLS) requests.
# CUSTOMIZE this section if you have additional packagers
director hls hash {
        {.backend = b1; .weight = 1;}
        {.backend = b2; .weight = 1;}
}
 
# define the round_robin director that is used for HDS requests.
# CUSTOMIZE this section if you have additional packagers
director hds round-robin {
    {.backend = b1;}
    {.backend = b2;}
}

## Called when a client request is received
sub vcl_recv {
	
	set req.backend = hds;
	
	## Use URL hashing direction scheme for .m3u8s in order to mitigate liveness issues on packager failure and restart 
	if (req.request == "GET" && req.url ~ "\.m3u8$" && req.url ~ "hls-live") {
		set req.backend = hls;
		if(req.restarts > 0){
			set req.url = req.url + "?restart=" + req.restarts;
		}
	}
	
	## Add a unique header containing the client address
	## for the purposes of logging
	if (req.restarts == 0) {
        if (req.http.x-forwarded-for) {
            set req.http.X-Forwarded-For =
                req.http.X-Forwarded-For + ", " + client.ip;
        } else {
            set req.http.X-Forwarded-For = client.ip;
        }
	}

	# grace settings, note this is also set in vcl_fetch,
	set req.grace = 80s;


	## always cache bootstraps:	
	if (req.request == "GET" && req.url ~ "\.(bootstrap)") {
		return(lookup);
	}

	## always cache fragments:
	if (req.request == "GET" && req.url ~ "(\wSeg[0-9]*-Frag[0-9]*)") {
		return(lookup);
	}
    
	## do not cache these rules:
    if (req.request != "GET" && req.request != "HEAD") {
		return(pipe);
	}
	
	# Handle compression correctly. Different browsers send different
	# "Accept-Encoding" headers, even though they mostly all support the same
	# compression mechanisms. By consolidating these compression headers into
	# a consistent format, we can reduce the size of the cache and get more hits.	 
	if (req.http.Accept-Encoding) {
		if (req.url ~ "\.(ts|bootstrap)$" || req.url ~ "(\wSeg[0-9]*-Frag[0-9]*)") {
            # No point in compressing these
            remove req.http.Accept-Encoding;
			
        } elsif (req.http.Accept-Encoding ~ "gzip") {
            set req.http.Accept-Encoding = "gzip";
			
        } elsif (req.http.Accept-Encoding ~ "deflate" && req.http.user-agent !~ "MSIE") {
            set req.http.Accept-Encoding = "deflate";
			
        } else {
            # unkown algorithm
            remove req.http.Accept-Encoding;
        }
	}

	## if it passes all these tests, do a lookup anyway;
	return(lookup);
}

## Called when the requested object has been retrieved from the backend, or the request to the backend has failed
sub vcl_fetch {
		
	# fail-over to the next server when we receive a 503. 
    # CUSTOMIZE the value of "503" for your deployment. It should be the same as the unavailable Response code set in httpd.conf at the server
    if (beresp.status == 503) {
        # CUSTOMIZE the value "1" for your deployment. it should be the number of packagers - 1.
        if(req.restarts < 1) {
            return (restart);
        } else {
            # all servers failed, generate a cache-able 404 error.
            # NOTE: we have chosen not to use the varnish
            # "error <number> <response>" mechanism to generate our
            # error since those errors are not cached by varnish.
            # instead, we will transform our 503 error into a 404 error.
            set beresp.status = 404;
            set beresp.response = "Not found.";
 
            # CUSTOMIZE ttl to be 1/2 a fragment interval.
            set beresp.ttl = 2s;

			# CUSTOMIZE Cache-Control to be 1/2 a fragment interval.
			set beresp.http.Cache-Control = "maxage=2";
            unset beresp.http.expires;
        }
    }
 
    set beresp.grace = 600s;  # If the backend is unreachable, hold content for 10 mins
 
    if (req.backend.healthy) {
        # Blanket cache everything for 1 fragment interval. CUSTOMIZE the value "4" for your deployment as the configured fragment duration.
        set beresp.ttl = 4s;
    } else {
        set beresp.ttl = 600s; # The backend is sick set ttl to 10 mins to serve stale content
    }
 
    ## Do not cache 50x errors
    if (beresp.status >= 500) { 
        set beresp.ttl = 0s; 
    }

	## Uncomment the below piece of code to achieve more fine-grained caching
	## Cache Live bootstraps for 1/2 a standard fragment duration
	# if (req.request == "GET" && req.url ~ "\.bootstrap$") {
	#	set beresp.ttl = 2s;	#1/2 frag_duration
	# }

	## Cache VoD Manifests for 1 day
	# if (req.request == "GET" && req.url ~ "\.f4m$" && req.url ~ "hds-vod") {
	#	set beresp.ttl = 86400s;# 1 day
	# }
	
	## Cache Live Manifests for 1/2 of standard fragment duration
	# if (req.request == "GET" && req.url ~ "\.f4m$" && req.url ~ "hds-live") {
	#	set beresp.ttl = 2s;	# 1/2 frag_duration
	#}
	
	## Cache Live M3U8s for 1/2 of standard TS duration
	# if (req.request == "GET" && req.url ~ "\.m3u8$" && req.url ~ "hls-live") {
	#	set beresp.ttl = 4s;	# 1/2 TS duration
	#}
	
	## Cache HDS/HLS VoD and Live fragments for 1 day
	# if (req.request == "GET" && (req.url ~ "\wSeg[0-9]*-Frag[0-9]*" || req.url ~ "\.ts$")) {
	#	set beresp.ttl = 86400s;# 1 day
	#}
	
	## Cache HEAD requests
	if (req.request != "GET" && req.request != "HEAD") {
		set beresp.ttl = 2s;	# 1/2 frag_duration
	}
	
    set beresp.http.X-Cacheable = "YES";
 
    return(deliver);
}

## If no packager contains 
sub vcl_error {
	# Add your logic here
}

## Called when an object is in the cache, its a hit.
sub vcl_hit {
	if (obj.ttl == 0s) {
		return(pass);
	}

	return(deliver);
}

## Called when the requested object was not found in the cache
sub vcl_miss {
	# Add your logic here
}

## Called before a cached object is delivered to the client
sub vcl_deliver {
	set resp.http.X-Served-By = server.hostname;
	if (obj.hits > 0) {
		set resp.http.X-Cache = "HIT";
	set resp.http.X-Cache-Hits = obj.hits;
	} else {
		set resp.http.X-Cache = "MISS";
 	}
	return(deliver);
}